"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const file_api_driver_joplinServer_1 = require("./file-api-driver-joplinServer");
const Setting_1 = require("./models/Setting");
const Synchronizer_1 = require("./Synchronizer");
const locale_js_1 = require("./locale.js");
const JoplinServerApi2_1 = require("./JoplinServerApi2");
const BaseSyncTarget = require('./BaseSyncTarget.js');
const { FileApi } = require('./file-api.js');
class SyncTargetJoplinServer extends BaseSyncTarget {
    static id() {
        return 9;
    }
    static supportsConfigCheck() {
        return true;
    }
    static targetName() {
        return 'joplinServer';
    }
    static label() {
        return locale_js_1._('Joplin Server');
    }
    isAuthenticated() {
        return __awaiter(this, void 0, void 0, function* () {
            return true;
        });
    }
    static newFileApi_(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const apiOptions = {
                baseUrl: () => options.path(),
                username: () => options.username(),
                password: () => options.password(),
            };
            const api = new JoplinServerApi2_1.default(apiOptions);
            const driver = new file_api_driver_joplinServer_1.default(api);
            const fileApi = new FileApi(() => `root:/${options.directory()}`, driver);
            fileApi.setSyncTargetId(this.id());
            yield fileApi.initialize();
            return fileApi;
        });
    }
    static checkConfig(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const fileApi = yield SyncTargetJoplinServer.newFileApi_(options);
            fileApi.requestRepeatCount_ = 0;
            const output = {
                ok: false,
                errorMessage: '',
            };
            try {
                const result = yield fileApi.stat('');
                if (!result)
                    throw new Error(`Sync directory not found: "${options.directory()}" on server "${options.path()}"`);
                output.ok = true;
            }
            catch (error) {
                output.errorMessage = error.message;
                if (error.code)
                    output.errorMessage += ` (Code ${error.code})`;
            }
            return output;
        });
    }
    initFileApi() {
        return __awaiter(this, void 0, void 0, function* () {
            const fileApi = yield SyncTargetJoplinServer.newFileApi_({
                path: () => Setting_1.default.value('sync.9.path'),
                username: () => Setting_1.default.value('sync.9.username'),
                password: () => Setting_1.default.value('sync.9.password'),
                directory: () => Setting_1.default.value('sync.9.directory'),
            });
            fileApi.setLogger(this.logger());
            return fileApi;
        });
    }
    initSynchronizer() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Synchronizer_1.default(this.db(), yield this.fileApi(), Setting_1.default.value('appType'));
        });
    }
}
exports.default = SyncTargetJoplinServer;
//# sourceMappingURL=SyncTargetJoplinServer.js.map