"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const contextkey_1 = require("./contextkey/contextkey");
class WhenClause {
    constructor(expression, validate) {
        this.rules_ = null;
        this.expression_ = expression;
        this.validate_ = validate;
    }
    createContext(ctx) {
        return {
            getValue: (key) => {
                return ctx[key];
            },
        };
    }
    get rules() {
        if (!this.rules_) {
            this.rules_ = contextkey_1.ContextKeyExpr.deserialize(this.expression_);
        }
        return this.rules_;
    }
    evaluate(context) {
        if (this.validate_)
            this.validate(context);
        return this.rules.evaluate(this.createContext(context));
    }
    validate(context) {
        const keys = this.rules.keys();
        for (const key of keys) {
            if (!(key in context))
                throw new Error(`No such key: ${key}`);
        }
    }
}
exports.default = WhenClause;
//# sourceMappingURL=WhenClause.js.map