"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const InteropService_Exporter_Base = require('./InteropService_Exporter_Base').default;
const { basename, friendlySafeFilename, rtrimSlashes } = require('../../path-utils');
const BaseModel = require('../../BaseModel').default;
const Folder = require('../../models/Folder');
const Note = require('../../models/Note');
const Setting = require('../../models/Setting').default;
const shim = require('../../shim').default;
const { themeStyle } = require('../../theme');
const { dirname } = require('../../path-utils');
const { escapeHtml } = require('../../string-utils.js');
const markupLanguageUtils = require('../../markupLanguageUtils').default;
const { assetsToHeaders } = require('@joplin/renderer');
class InteropService_Exporter_Html extends InteropService_Exporter_Base {
    init(path, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            this.customCss_ = options.customCss ? options.customCss : '';
            if (this.metadata().target === 'file') {
                this.destDir_ = dirname(path);
                this.filePath_ = path;
            }
            else {
                this.destDir_ = path;
                this.filePath_ = null;
            }
            this.createdDirs_ = [];
            this.resourceDir_ = this.destDir_ ? `${this.destDir_}/_resources` : null;
            yield shim.fsDriver().mkdir(this.destDir_);
            this.markupToHtml_ = markupLanguageUtils.newMarkupToHtml(options.plugins);
            this.resources_ = [];
            this.style_ = themeStyle(Setting.THEME_LIGHT);
        });
    }
    makeDirPath_(item, pathPart = null) {
        return __awaiter(this, void 0, void 0, function* () {
            let output = '';
            while (true) {
                if (item.type_ === BaseModel.TYPE_FOLDER) {
                    if (pathPart) {
                        output = `${pathPart}/${output}`;
                    }
                    else {
                        output = `${friendlySafeFilename(item.title, null, true)}/${output}`;
                        output = yield shim.fsDriver().findUniqueFilename(output);
                    }
                }
                if (!item.parent_id)
                    return output;
                item = yield Folder.load(item.parent_id);
            }
        });
    }
    processNoteResources_(item) {
        return __awaiter(this, void 0, void 0, function* () {
            const target = this.metadata().target;
            const linkedResourceIds = yield Note.linkedResourceIds(item.body);
            const relativePath = target === 'directory' ? rtrimSlashes(yield this.makeDirPath_(item, '..')) : '';
            const resourcePaths = this.context() && this.context().resourcePaths ? this.context().resourcePaths : {};
            let newBody = item.body;
            for (let i = 0; i < linkedResourceIds.length; i++) {
                const id = linkedResourceIds[i];
                const resourceContent = `${relativePath ? `${relativePath}/` : ''}_resources/${basename(resourcePaths[id])}`;
                newBody = newBody.replace(new RegExp(`:/${id}`, 'g'), resourceContent);
            }
            return newBody;
        });
    }
    processItem(_itemType, item) {
        return __awaiter(this, void 0, void 0, function* () {
            if ([BaseModel.TYPE_NOTE, BaseModel.TYPE_FOLDER].indexOf(item.type_) < 0)
                return;
            let dirPath = '';
            if (!this.filePath_) {
                dirPath = `${this.destDir_}/${yield this.makeDirPath_(item)}`;
                if (this.createdDirs_.indexOf(dirPath) < 0) {
                    yield shim.fsDriver().mkdir(dirPath);
                    this.createdDirs_.push(dirPath);
                }
            }
            if (item.type_ === BaseModel.TYPE_NOTE) {
                let noteFilePath = '';
                if (this.filePath_) {
                    noteFilePath = this.filePath_;
                }
                else {
                    noteFilePath = `${dirPath}/${friendlySafeFilename(item.title, null, true)}.html`;
                    noteFilePath = yield shim.fsDriver().findUniqueFilename(noteFilePath);
                }
                const bodyMd = yield this.processNoteResources_(item);
                const result = yield this.markupToHtml_.render(item.markup_language, bodyMd, this.style_, {
                    resources: this.resources_,
                    plainResourceRendering: true,
                    userCss: this.customCss_,
                });
                const noteContent = [];
                if (item.title)
                    noteContent.push(`<div class="exported-note-title">${escapeHtml(item.title)}</div>`);
                if (result.html)
                    noteContent.push(result.html);
                const libRootPath = dirname(dirname(__dirname));
                // We need to export all the plugin assets too and refer them from the header
                // The source path is a bit hard-coded but shouldn't change.
                for (let i = 0; i < result.pluginAssets.length; i++) {
                    const asset = result.pluginAssets[i];
                    const filePath = `${libRootPath}/node_modules/@joplin/renderer/assets/${asset.name}`;
                    const destPath = `${dirname(noteFilePath)}/pluginAssets/${asset.name}`;
                    yield shim.fsDriver().mkdir(dirname(destPath));
                    yield shim.fsDriver().copy(filePath, destPath);
                }
                const fullHtml = `
				<!DOCTYPE html>
				<html>
					<head>
						<meta charset="UTF-8">
						${assetsToHeaders(result.pluginAssets, { asHtml: true })}
						<title>${escapeHtml(item.title)}</title>
					</head>
					<body>
						<div class="exported-note">${noteContent.join('\n\n')}</div>
					</body>
				</html>
			`;
                yield shim.fsDriver().writeFile(noteFilePath, fullHtml, 'utf-8');
            }
        });
    }
    processResource(resource, filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const destResourcePath = `${this.resourceDir_}/${basename(filePath)}`;
            yield shim.fsDriver().copy(filePath, destResourcePath);
            this.resources_.push(resource);
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
}
exports.default = InteropService_Exporter_Html;
//# sourceMappingURL=InteropService_Exporter_Html.js.map