"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const InteropService_Exporter_Base = require('./InteropService_Exporter_Base').default;
const { basename, dirname, friendlySafeFilename } = require('../../path-utils');
const BaseModel = require('../../BaseModel').default;
const Folder = require('../../models/Folder');
const Note = require('../../models/Note');
const shim = require('../../shim').default;
const markdownUtils = require('../../markdownUtils').default;
class InteropService_Exporter_Md extends InteropService_Exporter_Base {
    init(destDir) {
        return __awaiter(this, void 0, void 0, function* () {
            this.destDir_ = destDir;
            this.resourceDir_ = destDir ? `${destDir}/_resources` : null;
            this.createdDirs_ = [];
            yield shim.fsDriver().mkdir(this.destDir_);
            yield shim.fsDriver().mkdir(this.resourceDir_);
        });
    }
    makeDirPath_(item, pathPart = null, findUniqueFilename = true) {
        return __awaiter(this, void 0, void 0, function* () {
            let output = '';
            while (true) {
                if (item.type_ === BaseModel.TYPE_FOLDER) {
                    if (pathPart) {
                        output = `${pathPart}/${output}`;
                    }
                    else {
                        output = `${friendlySafeFilename(item.title, null, true)}/${output}`;
                        if (findUniqueFilename)
                            output = yield shim.fsDriver().findUniqueFilename(output);
                    }
                }
                if (!item.parent_id)
                    return output;
                item = yield Folder.load(item.parent_id);
            }
        });
    }
    relaceLinkedItemIdsByRelativePaths_(item) {
        return __awaiter(this, void 0, void 0, function* () {
            const relativePathToRoot = yield this.makeDirPath_(item, '..');
            const newBody = yield this.replaceResourceIdsByRelativePaths_(item.body, relativePathToRoot);
            return yield this.replaceNoteIdsByRelativePaths_(newBody, relativePathToRoot);
        });
    }
    replaceResourceIdsByRelativePaths_(noteBody, relativePathToRoot) {
        return __awaiter(this, void 0, void 0, function* () {
            const linkedResourceIds = yield Note.linkedResourceIds(noteBody);
            const resourcePaths = this.context() && this.context().resourcePaths ? this.context().resourcePaths : {};
            const createRelativePath = function (resourcePath) {
                return `${relativePathToRoot}_resources/${basename(resourcePath)}`;
            };
            return yield this.replaceItemIdsByRelativePaths_(noteBody, linkedResourceIds, resourcePaths, createRelativePath);
        });
    }
    replaceNoteIdsByRelativePaths_(noteBody, relativePathToRoot) {
        return __awaiter(this, void 0, void 0, function* () {
            const linkedNoteIds = yield Note.linkedNoteIds(noteBody);
            const notePaths = this.context() && this.context().notePaths ? this.context().notePaths : {};
            const createRelativePath = function (notePath) {
                return markdownUtils.escapeLinkUrl(`${relativePathToRoot}${notePath}`.trim());
            };
            return yield this.replaceItemIdsByRelativePaths_(noteBody, linkedNoteIds, notePaths, createRelativePath);
        });
    }
    replaceItemIdsByRelativePaths_(noteBody, linkedItemIds, paths, fn_createRelativePath) {
        return __awaiter(this, void 0, void 0, function* () {
            let newBody = noteBody;
            for (let i = 0; i < linkedItemIds.length; i++) {
                const id = linkedItemIds[i];
                const itemPath = fn_createRelativePath(paths[id]);
                newBody = newBody.replace(new RegExp(`:/${id}`, 'g'), itemPath);
            }
            return newBody;
        });
    }
    prepareForProcessingItemType(itemType, itemsToExport) {
        return __awaiter(this, void 0, void 0, function* () {
            if (itemType === BaseModel.TYPE_NOTE) {
                // Create unique file path for the note
                const context = {
                    notePaths: {},
                };
                for (let i = 0; i < itemsToExport.length; i++) {
                    const itemType = itemsToExport[i].type;
                    if (itemType !== itemType)
                        continue;
                    const itemOrId = itemsToExport[i].itemOrId;
                    const note = typeof itemOrId === 'object' ? itemOrId : yield Note.load(itemOrId);
                    if (!note)
                        continue;
                    let notePath = `${yield this.makeDirPath_(note, null, false)}${friendlySafeFilename(note.title, null, true)}.md`;
                    notePath = yield shim.fsDriver().findUniqueFilename(`${this.destDir_}/${notePath}`, Object.values(context.notePaths));
                    context.notePaths[note.id] = notePath;
                }
                // Strip the absolute path to export dir and keep only the relative paths
                const destDir = this.destDir_;
                Object.keys(context.notePaths).map(function (id) {
                    context.notePaths[id] = context.notePaths[id].substr(destDir.length + 1);
                });
                this.updateContext(context);
            }
        });
    }
    processItem(_itemType, item) {
        return __awaiter(this, void 0, void 0, function* () {
            if ([BaseModel.TYPE_NOTE, BaseModel.TYPE_FOLDER].indexOf(item.type_) < 0)
                return;
            if (item.type_ === BaseModel.TYPE_FOLDER) {
                const dirPath = `${this.destDir_}/${yield this.makeDirPath_(item)}`;
                if (this.createdDirs_.indexOf(dirPath) < 0) {
                    yield shim.fsDriver().mkdir(dirPath);
                    this.createdDirs_.push(dirPath);
                }
            }
            else if (item.type_ === BaseModel.TYPE_NOTE) {
                const notePaths = this.context() && this.context().notePaths ? this.context().notePaths : {};
                const noteFilePath = `${this.destDir_}/${notePaths[item.id]}`;
                const noteBody = yield this.relaceLinkedItemIdsByRelativePaths_(item);
                const modNote = Object.assign({}, item, { body: noteBody });
                const noteContent = yield Note.serializeForEdit(modNote);
                yield shim.fsDriver().mkdir(dirname(noteFilePath));
                yield shim.fsDriver().writeFile(noteFilePath, noteContent, 'utf-8');
            }
        });
    }
    processResource(_resource, filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const destResourcePath = `${this.resourceDir_}/${basename(filePath)}`;
            yield shim.fsDriver().copy(filePath, destResourcePath);
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
}
exports.default = InteropService_Exporter_Md;
//# sourceMappingURL=InteropService_Exporter_Md.js.map