"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const locale_1 = require("../../locale");
const InteropService_Importer_Base = require('./InteropService_Importer_Base').default;
const Folder = require('../../models/Folder.js');
const Note = require('../../models/Note.js');
const { basename, filename, rtrimSlashes, fileExtension, dirname } = require('../../path-utils');
const shim = require('../../shim').default;
const { extractImageUrls } = require('../../markdownUtils').default;
const { unique } = require('../../ArrayUtils');
const { pregQuote } = require('../../string-utils-common');
const { MarkupToHtml } = require('@joplin/renderer');
class InteropService_Importer_Md extends InteropService_Importer_Base {
    exec(result) {
        return __awaiter(this, void 0, void 0, function* () {
            let parentFolderId = null;
            const sourcePath = rtrimSlashes(this.sourcePath_);
            const filePaths = [];
            if (yield shim.fsDriver().isDirectory(sourcePath)) {
                if (!this.options_.destinationFolder) {
                    const folderTitle = yield Folder.findUniqueItemTitle(basename(sourcePath));
                    const folder = yield Folder.save({ title: folderTitle });
                    parentFolderId = folder.id;
                }
                else {
                    parentFolderId = this.options_.destinationFolder.id;
                }
                yield this.importDirectory(sourcePath, parentFolderId);
            }
            else {
                if (!this.options_.destinationFolder)
                    throw new Error(locale_1._('Please specify the notebook where the notes should be imported to.'));
                parentFolderId = this.options_.destinationFolder.id;
                filePaths.push(sourcePath);
            }
            for (let i = 0; i < filePaths.length; i++) {
                yield this.importFile(filePaths[i], parentFolderId);
            }
            return result;
        });
    }
    importDirectory(dirPath, parentFolderId) {
        return __awaiter(this, void 0, void 0, function* () {
            console.info(`Import: ${dirPath}`);
            const supportedFileExtension = this.metadata().fileExtensions;
            const stats = yield shim.fsDriver().readDirStats(dirPath);
            for (let i = 0; i < stats.length; i++) {
                const stat = stats[i];
                if (stat.isDirectory()) {
                    const folderTitle = yield Folder.findUniqueItemTitle(basename(stat.path));
                    const folder = yield Folder.save({ title: folderTitle, parent_id: parentFolderId });
                    yield this.importDirectory(`${dirPath}/${basename(stat.path)}`, folder.id);
                }
                else if (supportedFileExtension.indexOf(fileExtension(stat.path).toLowerCase()) >= 0) {
                    yield this.importFile(`${dirPath}/${stat.path}`, parentFolderId);
                }
            }
        });
    }
    /**
     * Parse text for links, attempt to find local file, if found create Joplin resource
     * and update link accordingly.
     */
    importLocalImages(filePath, md) {
        return __awaiter(this, void 0, void 0, function* () {
            let updated = md;
            const imageLinks = unique(extractImageUrls(md));
            yield Promise.all(imageLinks.map((encodedLink) => __awaiter(this, void 0, void 0, function* () {
                const link = decodeURI(encodedLink);
                const attachmentPath = filename(`${dirname(filePath)}/${link}`, true);
                const pathWithExtension = `${attachmentPath}.${fileExtension(link)}`;
                const stat = yield shim.fsDriver().stat(pathWithExtension);
                const isDir = stat ? stat.isDirectory() : false;
                if (stat && !isDir) {
                    const resource = yield shim.createResourceFromPath(pathWithExtension);
                    // NOTE: use ](link) in case the link also appears elsewhere, such as in alt text
                    const linkPatternEscaped = pregQuote(`](${link})`);
                    const reg = new RegExp(linkPatternEscaped, 'g');
                    updated = updated.replace(reg, `](:/${resource.id})`);
                }
            })));
            return updated;
        });
    }
    importFile(filePath, parentFolderId) {
        return __awaiter(this, void 0, void 0, function* () {
            const stat = yield shim.fsDriver().stat(filePath);
            if (!stat)
                throw new Error(`Cannot read ${filePath}`);
            const title = filename(filePath);
            const body = yield shim.fsDriver().readFile(filePath);
            let updatedBody;
            try {
                updatedBody = yield this.importLocalImages(filePath, body);
            }
            catch (error) {
                // console.error(`Problem importing links for file ${filePath}, error:\n ${error}`);
            }
            const note = {
                parent_id: parentFolderId,
                title: title,
                body: updatedBody || body,
                updated_time: stat.mtime.getTime(),
                created_time: stat.birthtime.getTime(),
                user_updated_time: stat.mtime.getTime(),
                user_created_time: stat.birthtime.getTime(),
                markup_language: MarkupToHtml.MARKUP_LANGUAGE_MARKDOWN,
            };
            return Note.save(note, { autoTimestamp: false });
        });
    }
}
exports.default = InteropService_Importer_Md;
//# sourceMappingURL=InteropService_Importer_Md.js.map