"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const shim_1 = require("../../shim");
const md5 = require('md5');
class RepositoryApi {
    constructor(baseUrl, tempDir) {
        this.manifests_ = null;
        this.baseUrl_ = baseUrl;
        this.tempDir_ = tempDir;
    }
    get contentBaseUrl() {
        return `${this.baseUrl_.replace(/github\.com/, 'raw.githubusercontent.com')}/master`;
    }
    fileUrl(relativePath) {
        return `${this.contentBaseUrl}/${relativePath}`;
    }
    fetchText(path) {
        return __awaiter(this, void 0, void 0, function* () {
            return shim_1.default.fetchText(this.fileUrl(path));
        });
    }
    search(query) {
        return __awaiter(this, void 0, void 0, function* () {
            query = query.toLowerCase().trim();
            const manifests = yield this.manifests();
            const output = [];
            for (const manifest of manifests) {
                for (const field of ['name', 'description']) {
                    const v = manifest[field];
                    if (!v)
                        continue;
                    if (v.toLowerCase().indexOf(query) >= 0) {
                        output.push(manifest);
                        break;
                    }
                }
            }
            return output;
        });
    }
    // Returns a temporary path, where the plugin has been downloaded to. Temp
    // file should be deleted by caller.
    downloadPlugin(pluginId) {
        return __awaiter(this, void 0, void 0, function* () {
            const manifests = yield this.manifests();
            const manifest = manifests.find(m => m.id === pluginId);
            if (!manifest)
                throw new Error(`No manifest for plugin ID "${pluginId}"`);
            const fileUrl = this.fileUrl(`plugins/${manifest.id}/plugin.jpl`);
            const hash = md5(Date.now() + Math.random());
            const targetPath = `${this.tempDir_}/${hash}_${manifest.id}.jpl`;
            const response = yield shim_1.default.fetchBlob(fileUrl, {
                path: targetPath,
            });
            if (!response.ok)
                throw new Error(`Could not download plugin "${pluginId}" from "${fileUrl}"`);
            return targetPath;
        });
    }
    manifests() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.manifests_)
                return this.manifests_;
            const manifestsText = yield this.fetchText('manifests.json');
            try {
                const manifests = JSON.parse(manifestsText);
                if (!manifests)
                    throw new Error('Invalid or missing JSON');
                this.manifests_ = Object.keys(manifests).map(id => {
                    return manifests[id];
                });
                return this.manifests_;
            }
            catch (error) {
                throw new Error(`Could not parse JSON: ${error.message}`);
            }
        });
    }
}
exports.default = RepositoryApi;
//# sourceMappingURL=RepositoryApi.js.map